#include <klocalizedstring.h>

/********************************************************************************
** Form generated from reading UI file 'walletcontrolwidget.ui'
**
** Created by: Qt User Interface Compiler version 5.6.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WALLETCONTROLWIDGET_H
#define UI_WALLETCONTROLWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ksqueezedtextlabel.h"

QT_BEGIN_NAMESPACE

class Ui_WalletControlWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    KSqueezedTextLabel *_stateLabel;
    QPushButton *_openClose;
    QSpacerItem *topSpacer;
    QPushButton *_changePassword;
    QTabWidget *_tabs;
    QWidget *_contentsTab;
    QHBoxLayout *horizontalLayout_2;
    QFrame *_editorFrame;
    QVBoxLayout *_editorFrameLayout;
    QWidget *_appsTab;
    QVBoxLayout *verticalLayout;
    QFrame *_applicationsFrame;
    QVBoxLayout *_applicationsFrameLayout;

    void setupUi(QWidget *WalletControlWidget)
    {
        if (WalletControlWidget->objectName().isEmpty())
            WalletControlWidget->setObjectName(QStringLiteral("WalletControlWidget"));
        WalletControlWidget->resize(700, 700);
        verticalLayout_2 = new QVBoxLayout(WalletControlWidget);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName(QStringLiteral("verticalLayout_2"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QStringLiteral("horizontalLayout"));
        _stateLabel = new KSqueezedTextLabel(WalletControlWidget);
        _stateLabel->setObjectName(QStringLiteral("_stateLabel"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_stateLabel->sizePolicy().hasHeightForWidth());
        _stateLabel->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(_stateLabel);

        _openClose = new QPushButton(WalletControlWidget);
        _openClose->setObjectName(QStringLiteral("_openClose"));

        horizontalLayout->addWidget(_openClose);

        topSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(topSpacer);

        _changePassword = new QPushButton(WalletControlWidget);
        _changePassword->setObjectName(QStringLiteral("_changePassword"));

        horizontalLayout->addWidget(_changePassword);


        verticalLayout_2->addLayout(horizontalLayout);

        _tabs = new QTabWidget(WalletControlWidget);
        _tabs->setObjectName(QStringLiteral("_tabs"));
        _contentsTab = new QWidget();
        _contentsTab->setObjectName(QStringLiteral("_contentsTab"));
        horizontalLayout_2 = new QHBoxLayout(_contentsTab);
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2->setObjectName(QStringLiteral("horizontalLayout_2"));
        _editorFrame = new QFrame(_contentsTab);
        _editorFrame->setObjectName(QStringLiteral("_editorFrame"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_editorFrame->sizePolicy().hasHeightForWidth());
        _editorFrame->setSizePolicy(sizePolicy1);
        _editorFrame->setFrameShape(QFrame::NoFrame);
        _editorFrame->setFrameShadow(QFrame::Plain);
        _editorFrame->setLineWidth(0);
        _editorFrameLayout = new QVBoxLayout(_editorFrame);
        _editorFrameLayout->setContentsMargins(0, 0, 0, 0);
        _editorFrameLayout->setObjectName(QStringLiteral("_editorFrameLayout"));

        horizontalLayout_2->addWidget(_editorFrame);

        QIcon icon;
        QString iconThemeName = QStringLiteral("wallet-closed");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QStringLiteral(""), QSize(), QIcon::Normal, QIcon::Off);
        }
        _tabs->addTab(_contentsTab, icon, QString());
        _appsTab = new QWidget();
        _appsTab->setObjectName(QStringLiteral("_appsTab"));
        verticalLayout = new QVBoxLayout(_appsTab);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName(QStringLiteral("verticalLayout"));
        _applicationsFrame = new QFrame(_appsTab);
        _applicationsFrame->setObjectName(QStringLiteral("_applicationsFrame"));
        _applicationsFrame->setFrameShape(QFrame::NoFrame);
        _applicationsFrame->setFrameShadow(QFrame::Plain);
        _applicationsFrame->setLineWidth(0);
        _applicationsFrameLayout = new QVBoxLayout(_applicationsFrame);
        _applicationsFrameLayout->setContentsMargins(0, 0, 0, 0);
        _applicationsFrameLayout->setObjectName(QStringLiteral("_applicationsFrameLayout"));

        verticalLayout->addWidget(_applicationsFrame);

        QIcon icon1;
        iconThemeName = QStringLiteral("start-here-kde");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QStringLiteral(""), QSize(), QIcon::Normal, QIcon::Off);
        }
        _tabs->addTab(_appsTab, icon1, QString());

        verticalLayout_2->addWidget(_tabs);

        QWidget::setTabOrder(_openClose, _changePassword);
        QWidget::setTabOrder(_changePassword, _tabs);

        retranslateUi(WalletControlWidget);
        QObject::connect(_openClose, SIGNAL(clicked()), WalletControlWidget, SLOT(onOpenClose()));
        QObject::connect(_changePassword, SIGNAL(clicked()), WalletControlWidget, SLOT(onChangePassword()));

        _tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(WalletControlWidget);
    } // setupUi

    void retranslateUi(QWidget *WalletControlWidget)
    {
        _stateLabel->setText(tr2i18n("KSqueezedTextLabel", 0));
        _openClose->setText(tr2i18n("Open...", 0));
        _changePassword->setText(tr2i18n("Change Password...", 0));
        _tabs->setTabText(_tabs->indexOf(_contentsTab), tr2i18n("Contents", 0));
        _tabs->setTabText(_tabs->indexOf(_appsTab), tr2i18n("Applications", 0));
        Q_UNUSED(WalletControlWidget);
    } // retranslateUi

};

namespace Ui {
    class WalletControlWidget: public Ui_WalletControlWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // WALLETCONTROLWIDGET_H

